/*
 *  image viewer program for FreyaBIOS functionality check
 *	issued: 2000/4/25
 *	author: c.mos
 */

#define _DEBUG

#include "../std.h"
#include "../biosinl.h"
#include "../resource.h"
#include <sys/fcntl.h>
#include <sys/process.h>
#include <string.h>
#include <stdlib.h>

#define LCD_CHAR_CX	28
#define LCD_CHAR_CY	18

#define PAL_IMAGE 0
#define VBLANK_FREQ 75

extern BYTE *_heap;

BOOL select_file(char far *path);
size_t read_file(char *path, void *buffer);
void view_image(res_image_t* res, int home_y);
int atoi(char *s);

void main(int argc, char *argv[])
{
	fent_t fent;
	static char pathbuff[MAXPATHLEN];
	char *path = argv[1];
	res_image_t *res = (res_image_t*)_heap;
	int home_x, home_y = 0;
	int wait = 0;

	text_screen_init();
	if(argc < 2) {
		if(!select_file(pathbuff)) goto Exit;
		path = pathbuff;
	}
	if(!read_file(path, _heap)) goto Exit;
	if(res->header.type != RESID_IMAGE) {
		text_put_string(0, 0, "it's not image file");
		key_wait();
	} else {
		if(argc >= 3) {
			wait = (atoi(argv[2]) * VBLANK_FREQ) / 10; /* 1/10sec */
		}
		if(wait) {
			long tick = sys_get_tick_count();
			view_image(res, 0);
			do {
				if(key_hit_check_with_repeat()) break;
			} while(sys_get_tick_count() < tick + wait);
		} else {
			for(;;) {
				view_image(res, home_y);
				switch(key_wait()) {
					case KEY_UP1:
						if(home_y) home_y--;
						break;
					case KEY_DOWN1:
						if(home_y + LCD_CHAR_CY < (res->height)) home_y++;
						break;
					case KEY_LEFT1:
					case KEY_RIGHT1:
						break;
					default:
						goto Exit;
				}
				text_screen_init();
			}
		}
	}
Exit:;
}

void view_image(res_image_t* res, int home_y)
{
	int x, y;
	int cx = min(res->width, LCD_CHAR_CX);
	int cy = min(res->height, LCD_CHAR_CY);
	int fontbytes = (res->format == IMAGE_MONO) ? 8 : 16;
	BYTE* bits = (BYTE*)(res + 1);
	bits += res->width * home_y * fontbytes;
	if(res->format != IMAGE_MONO) {
		WORD color = 0;
		int i;
		for(i=3; i>=0; i--)
			color = (color << 4) | (res->clrtable[i] >> 5);	 /* ## 00.7.23 */
		palette_set_color(PAL_IMAGE, color);
	}
	for(y=0; y < cy; y++) {
		for(x=0; x < cx; x++) {
			WORD code = screen_get_char1(SCREEN2, x, y) & CFM_FONT;
			if(res->format == IMAGE_MONO)
				font_set_monodata(code, 1, bits);
			else
				font_set_colordata(code, 1, bits);
			bits += fontbytes;
		}
	}
}

int atoi(char *s)
{
	int val = 0;
	char c;
	while(c = *s++) {
		if(c < '0' || c > '9') break;
		val = val * 10 + (c - '0');
	}
	return val;
}
